#region UICode
double Amount1 = 45; // [-180,180] Rotation
byte Amount2 = 0; // [1] Limit Type|Clamp|Wrap|Mirror|Primary Colour|Secondary Colour|Untouch
bool Amount3 = false; // [0,1] Disable Interpolation
#endregion

// Written by Stuart Radforth (paintnet@radforth.com)
// Simple rotation tool

// Function to sample 4 texels and interpolate them
ColorBgra Interpolate( Surface src, Rectangle selection, double tempX, double tempY )
{
    ColorBgra CurrentPixel;
    ColorBgra CurrentPixel00;
    ColorBgra CurrentPixel01;
    ColorBgra CurrentPixel10;
    ColorBgra CurrentPixel11;
    double valueR;
    double valueG;
    double valueB;
    double valueA;
    double blendX0;
    double blendY0;
    double blendX1;
    double blendY1;
    double tempX2;
    double tempY2;
    
    CurrentPixel = src[ (int)tempX, (int)tempY ];
    // If intepolation is disabled
    if( Amount3 )
    {
        CurrentPixel = src[ (int)tempX, (int)tempY ];
    }
    else
    {
        // Do bilinear interpolation
        tempX2 = tempX + 1.0;
        tempY2 = tempY + 1.0;
        if( (double)((int)tempX2) >= selection.Right )
        {
            tempX2 = selection.Right - 1.0;
        }
        if( (double)((int)tempY2) >= selection.Bottom )
        {
            tempY2 = selection.Bottom - 1.0;
        }

        // Sample the 4 texels
        CurrentPixel00 = src[(int)Math.Floor(tempX),  (int)Math.Floor(tempY)];
        CurrentPixel10 = src[(int)Math.Floor(tempX2), (int)Math.Floor(tempY)];
        CurrentPixel01 = src[(int)Math.Floor(tempX),  (int)Math.Floor(tempY2)];
        CurrentPixel11 = src[(int)Math.Floor(tempX2), (int)Math.Floor(tempY2)];
    
        // Blend values together
        blendX1 = tempX - Math.Floor(tempX);
        blendX0 = 1.0 - blendX1;
        blendY1 = tempY - Math.Floor(tempY);
        blendY0 = 1.0 - blendY1;
    
    
        // Blend 4 texels together and clamp in case there are rounding errors
        valueR = ((((((double)CurrentPixel00.R) * blendX0)) + (((double)CurrentPixel10.R) * blendX1)) * blendY0) +
                 ((((((double)CurrentPixel01.R) * blendX0)) + (((double)CurrentPixel11.R) * blendX1)) * blendY1);
        if( valueR > 255.0 )
        {
            valueR = 255.0;
        }
        else if ( valueR < 0.0 )
        {
            valueR = 0.0;
        }                        
    
        valueG = ((((double)CurrentPixel00.G) * blendX0) + ((double)CurrentPixel10.G) * blendX1) * blendY0 +
                 ((((double)CurrentPixel01.G) * blendX0) + ((double)CurrentPixel11.G) * blendX1) * blendY1;
        if( valueG > 255.0 )
        {
            valueG = 255.0;
        }
        else if ( valueG < 0.0 )
        {
            valueG = 0.0;
        }                        
    
        valueB = ((((double)CurrentPixel00.B) * blendX0) + ((double)CurrentPixel10.B) * blendX1) * blendY0 +
                 ((((double)CurrentPixel01.B) * blendX0) + ((double)CurrentPixel11.B) * blendX1) * blendY1;
        if( valueB > 255.0 )
        {
            valueB = 255.0;
        }
        else if ( valueB < 0.0 )
        {
            valueB = 0.0;
        }                        
    
        valueA = ((((double)CurrentPixel00.A) * blendX0) + ((double)CurrentPixel10.A) * blendX1) * blendY0 +
                 ((((double)CurrentPixel01.A) * blendX0) + ((double)CurrentPixel11.A) * blendX1) * blendY1;
        if( valueA > 255.0 )
        {
            valueA = 255.0;
        }
        else if ( valueA < 0.0 )
        {
            valueA = 0.0;
        }                        
    
        // Write blended value back to buffer
        CurrentPixel.R = (byte)valueR;
        CurrentPixel.G = (byte)valueG;
        CurrentPixel.B = (byte)valueB;
        CurrentPixel.A = (byte)valueA;
    }
    
    return CurrentPixel;
}


// Main function
void Render(Surface dst, Surface src, Rectangle rect)
{
    // Delete any of these lines you don't need
    Rectangle selection = this.EnvironmentParameters.GetSelection(src.Bounds).GetBoundsInt();
    long CenterX = (long)(((selection.Right - selection.Left) / 2)+selection.Left);
    long CenterY = (long)(((selection.Bottom - selection.Top) / 2)+selection.Top);
    ColorBgra PrimaryColor = (ColorBgra)EnvironmentParameters.PrimaryColor;
    ColorBgra SecondaryColor = (ColorBgra)EnvironmentParameters.SecondaryColor;
    int BrushWidth = (int)EnvironmentParameters.BrushWidth;

    ColorBgra CurrentPixel;
    double tempX;
    double tempY;
    bool bError;
    double dAngle;
    double dCos, dSin;
    dAngle = Amount1*6.283/360.0;
    dCos = Math.Cos(dAngle);
    dSin = Math.Sin(dAngle);

    for (int y = rect.Top; y < rect.Bottom; y++)
    {
        for (int x = rect.Left; x < rect.Right; x++)
        {   
            tempX = ((double)(x - CenterX)) * dCos + (((double)(y - CenterY)) * dSin) + CenterX;
            tempY = ((double)(y - CenterY)) * dCos + (((double)(x - CenterX)) * -1 * dSin) + CenterY;

            CurrentPixel = src[x,y];
            
            // Clip
            bError = false;
            if( tempX >= selection.Right )
            {
                bError = true;
            }
            else if( tempX < selection.Left )
            {
                bError = true;
            }
            if( tempY >= selection.Bottom )
            {
                bError = true;
            }
            else if( tempY < selection.Top )
            {
                bError = true;
            }
            
            // Copy the background colour if an error is encountered
            if( bError )
            {
                switch(Amount2) 
                {
                    // Clamp
                    case 0:
                        if( tempX >= selection.Right )
                        {
                            tempX = selection.Right - 1;
                        }
                        else if( tempX < selection.Left )
                        {
                            tempX = selection.Left;
                        }
                        if( tempY >= selection.Bottom )
                        {
                            tempY = selection.Bottom - 1;
                        }
                        else if( tempY < selection.Top )
                        {
                            tempY = selection.Top;
                        }
                        CurrentPixel = Interpolate( src, selection, tempX,tempY);
                        break;            
                    
                    // Wrap
                    case 1:
                        tempX = (((int)tempX + (selection.Right-selection.Left) ) % (selection.Right - selection.Left)) + selection.Left;
                        tempY = (((int)tempY + (selection.Bottom-selection.Top) ) % (selection.Bottom - selection.Top)) + selection.Top;
                        tempX = ((int)tempX)%selection.Right;
                        tempY = ((int)tempY)%selection.Bottom;
                        CurrentPixel = Interpolate( src, selection, tempX, tempY );
                        break;
                    // Mirror
                    case 2:
                    
                        double dSelectionWidth;
                        double dSelectionHeight;
                        dSelectionWidth = (selection.Right - selection.Left);
                        dSelectionHeight = (selection.Bottom - selection.Top);
                        
                        double dOffsetX = (tempX - selection.Left) + (dSelectionWidth * 10000.0);
                        double dOffsetY = (tempY - selection.Top) + (dSelectionHeight * 10000.0);
                    
                        dOffsetX %= (2.0*dSelectionWidth);
                        dOffsetY %= (2.0*dSelectionHeight);
                    
                        if( dOffsetX > (dSelectionWidth) )
                        {
                            dOffsetX = (dSelectionWidth * 2.0) - dOffsetX;
                        }
                        if( dOffsetY > (dSelectionHeight) )
                        {
                            dOffsetY = (dSelectionHeight * 2.0) - dOffsetY;
                        }
                    
                    
                        tempX = dOffsetX + selection.Left;
                        tempY = dOffsetY + selection.Top;
                        CurrentPixel = Interpolate( src, selection, tempX, tempY );
                    break;
                    // Set with primary colour
                    case 3:
                        CurrentPixel.R = (byte)PrimaryColor.R;
                        CurrentPixel.G = (byte)PrimaryColor.G;
                        CurrentPixel.B = (byte)PrimaryColor.B;
                        CurrentPixel.A = (byte)PrimaryColor.A;
                        break;
                    // Set with secondary colour
                    case 4:
                        CurrentPixel.R = (byte)SecondaryColor.R;
                        CurrentPixel.G = (byte)SecondaryColor.G;
                        CurrentPixel.B = (byte)SecondaryColor.B;
                        CurrentPixel.A = (byte)SecondaryColor.A;
                        break;
                    // Untouch Leave src pixels
                    case 5:
                    default:
                        break;
                };
            }
            else
            {
                CurrentPixel = Interpolate( src, selection, tempX, tempY);
            }
            dst[x,y] = CurrentPixel;
        }
    }
}
